"""
Type annotations for emr service type definitions.

[Open documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/type_defs.html)

Usage::

    ```python
    from types_aiobotocore_emr.type_defs import AddInstanceFleetInputRequestTypeDef

    data: AddInstanceFleetInputRequestTypeDef = {...}
    ```
"""
import sys
from datetime import datetime
from typing import Any, Dict, List, Mapping, Sequence, Union

from typing_extensions import NotRequired

from .literals import (
    ActionOnFailureType,
    AdjustmentTypeType,
    AuthModeType,
    AutoScalingPolicyStateChangeReasonCodeType,
    AutoScalingPolicyStateType,
    CancelStepsRequestStatusType,
    ClusterStateChangeReasonCodeType,
    ClusterStateType,
    ComparisonOperatorType,
    ComputeLimitsUnitTypeType,
    IdentityTypeType,
    InstanceCollectionTypeType,
    InstanceFleetStateChangeReasonCodeType,
    InstanceFleetStateType,
    InstanceFleetTypeType,
    InstanceGroupStateChangeReasonCodeType,
    InstanceGroupStateType,
    InstanceGroupTypeType,
    InstanceRoleTypeType,
    InstanceStateChangeReasonCodeType,
    InstanceStateType,
    JobFlowExecutionStateType,
    MarketTypeType,
    NotebookExecutionStatusType,
    OnDemandCapacityReservationPreferenceType,
    PlacementGroupStrategyType,
    RepoUpgradeOnBootType,
    ScaleDownBehaviorType,
    SpotProvisioningTimeoutActionType,
    StatisticType,
    StepCancellationOptionType,
    StepExecutionStateType,
    StepStateType,
    UnitType,
)

if sys.version_info >= (3, 9):
    from typing import Literal
else:
    from typing_extensions import Literal
if sys.version_info >= (3, 9):
    from typing import TypedDict
else:
    from typing_extensions import TypedDict


__all__ = (
    "AddInstanceFleetInputRequestTypeDef",
    "AddInstanceFleetOutputTypeDef",
    "AddInstanceGroupsInputRequestTypeDef",
    "AddInstanceGroupsOutputTypeDef",
    "AddJobFlowStepsInputRequestTypeDef",
    "AddJobFlowStepsOutputTypeDef",
    "AddTagsInputRequestTypeDef",
    "ApplicationTypeDef",
    "AutoScalingPolicyDescriptionTypeDef",
    "AutoScalingPolicyStateChangeReasonTypeDef",
    "AutoScalingPolicyStatusTypeDef",
    "AutoScalingPolicyTypeDef",
    "AutoTerminationPolicyTypeDef",
    "BlockPublicAccessConfigurationMetadataTypeDef",
    "BlockPublicAccessConfigurationTypeDef",
    "BootstrapActionConfigTypeDef",
    "BootstrapActionDetailTypeDef",
    "CancelStepsInfoTypeDef",
    "CancelStepsInputRequestTypeDef",
    "CancelStepsOutputTypeDef",
    "CloudWatchAlarmDefinitionTypeDef",
    "ClusterStateChangeReasonTypeDef",
    "ClusterStatusTypeDef",
    "ClusterSummaryTypeDef",
    "ClusterTimelineTypeDef",
    "ClusterTypeDef",
    "CommandTypeDef",
    "ComputeLimitsTypeDef",
    "ConfigurationTypeDef",
    "CreateSecurityConfigurationInputRequestTypeDef",
    "CreateSecurityConfigurationOutputTypeDef",
    "CreateStudioInputRequestTypeDef",
    "CreateStudioOutputTypeDef",
    "CreateStudioSessionMappingInputRequestTypeDef",
    "DeleteSecurityConfigurationInputRequestTypeDef",
    "DeleteStudioInputRequestTypeDef",
    "DeleteStudioSessionMappingInputRequestTypeDef",
    "DescribeClusterInputRequestTypeDef",
    "DescribeClusterOutputTypeDef",
    "DescribeJobFlowsInputRequestTypeDef",
    "DescribeJobFlowsOutputTypeDef",
    "DescribeNotebookExecutionInputRequestTypeDef",
    "DescribeNotebookExecutionOutputTypeDef",
    "DescribeReleaseLabelInputRequestTypeDef",
    "DescribeReleaseLabelOutputTypeDef",
    "DescribeSecurityConfigurationInputRequestTypeDef",
    "DescribeSecurityConfigurationOutputTypeDef",
    "DescribeStepInputRequestTypeDef",
    "DescribeStepOutputTypeDef",
    "DescribeStudioInputRequestTypeDef",
    "DescribeStudioOutputTypeDef",
    "EbsBlockDeviceConfigTypeDef",
    "EbsBlockDeviceTypeDef",
    "EbsConfigurationTypeDef",
    "EbsVolumeTypeDef",
    "Ec2InstanceAttributesTypeDef",
    "ExecutionEngineConfigTypeDef",
    "FailureDetailsTypeDef",
    "GetAutoTerminationPolicyInputRequestTypeDef",
    "GetAutoTerminationPolicyOutputTypeDef",
    "GetBlockPublicAccessConfigurationOutputTypeDef",
    "GetManagedScalingPolicyInputRequestTypeDef",
    "GetManagedScalingPolicyOutputTypeDef",
    "GetStudioSessionMappingInputRequestTypeDef",
    "GetStudioSessionMappingOutputTypeDef",
    "HadoopJarStepConfigTypeDef",
    "HadoopStepConfigTypeDef",
    "InstanceFleetConfigTypeDef",
    "InstanceFleetModifyConfigTypeDef",
    "InstanceFleetProvisioningSpecificationsTypeDef",
    "InstanceFleetStateChangeReasonTypeDef",
    "InstanceFleetStatusTypeDef",
    "InstanceFleetTimelineTypeDef",
    "InstanceFleetTypeDef",
    "InstanceGroupConfigTypeDef",
    "InstanceGroupDetailTypeDef",
    "InstanceGroupModifyConfigTypeDef",
    "InstanceGroupStateChangeReasonTypeDef",
    "InstanceGroupStatusTypeDef",
    "InstanceGroupTimelineTypeDef",
    "InstanceGroupTypeDef",
    "InstanceResizePolicyTypeDef",
    "InstanceStateChangeReasonTypeDef",
    "InstanceStatusTypeDef",
    "InstanceTimelineTypeDef",
    "InstanceTypeConfigTypeDef",
    "InstanceTypeDef",
    "InstanceTypeSpecificationTypeDef",
    "JobFlowDetailTypeDef",
    "JobFlowExecutionStatusDetailTypeDef",
    "JobFlowInstancesConfigTypeDef",
    "JobFlowInstancesDetailTypeDef",
    "KerberosAttributesTypeDef",
    "KeyValueTypeDef",
    "ListBootstrapActionsInputRequestTypeDef",
    "ListBootstrapActionsOutputTypeDef",
    "ListClustersInputRequestTypeDef",
    "ListClustersOutputTypeDef",
    "ListInstanceFleetsInputRequestTypeDef",
    "ListInstanceFleetsOutputTypeDef",
    "ListInstanceGroupsInputRequestTypeDef",
    "ListInstanceGroupsOutputTypeDef",
    "ListInstancesInputRequestTypeDef",
    "ListInstancesOutputTypeDef",
    "ListNotebookExecutionsInputRequestTypeDef",
    "ListNotebookExecutionsOutputTypeDef",
    "ListReleaseLabelsInputRequestTypeDef",
    "ListReleaseLabelsOutputTypeDef",
    "ListSecurityConfigurationsInputRequestTypeDef",
    "ListSecurityConfigurationsOutputTypeDef",
    "ListStepsInputRequestTypeDef",
    "ListStepsOutputTypeDef",
    "ListStudioSessionMappingsInputRequestTypeDef",
    "ListStudioSessionMappingsOutputTypeDef",
    "ListStudiosInputRequestTypeDef",
    "ListStudiosOutputTypeDef",
    "ManagedScalingPolicyTypeDef",
    "MetricDimensionTypeDef",
    "ModifyClusterInputRequestTypeDef",
    "ModifyClusterOutputTypeDef",
    "ModifyInstanceFleetInputRequestTypeDef",
    "ModifyInstanceGroupsInputRequestTypeDef",
    "NotebookExecutionSummaryTypeDef",
    "NotebookExecutionTypeDef",
    "OnDemandCapacityReservationOptionsTypeDef",
    "OnDemandProvisioningSpecificationTypeDef",
    "PaginatorConfigTypeDef",
    "PlacementGroupConfigTypeDef",
    "PlacementTypeTypeDef",
    "PortRangeTypeDef",
    "PutAutoScalingPolicyInputRequestTypeDef",
    "PutAutoScalingPolicyOutputTypeDef",
    "PutAutoTerminationPolicyInputRequestTypeDef",
    "PutBlockPublicAccessConfigurationInputRequestTypeDef",
    "PutManagedScalingPolicyInputRequestTypeDef",
    "ReleaseLabelFilterTypeDef",
    "RemoveAutoScalingPolicyInputRequestTypeDef",
    "RemoveAutoTerminationPolicyInputRequestTypeDef",
    "RemoveManagedScalingPolicyInputRequestTypeDef",
    "RemoveTagsInputRequestTypeDef",
    "ResponseMetadataTypeDef",
    "RunJobFlowInputRequestTypeDef",
    "RunJobFlowOutputTypeDef",
    "ScalingActionTypeDef",
    "ScalingConstraintsTypeDef",
    "ScalingRuleTypeDef",
    "ScalingTriggerTypeDef",
    "ScriptBootstrapActionConfigTypeDef",
    "SecurityConfigurationSummaryTypeDef",
    "SessionMappingDetailTypeDef",
    "SessionMappingSummaryTypeDef",
    "SetTerminationProtectionInputRequestTypeDef",
    "SetVisibleToAllUsersInputRequestTypeDef",
    "ShrinkPolicyTypeDef",
    "SimpleScalingPolicyConfigurationTypeDef",
    "SimplifiedApplicationTypeDef",
    "SpotProvisioningSpecificationTypeDef",
    "StartNotebookExecutionInputRequestTypeDef",
    "StartNotebookExecutionOutputTypeDef",
    "StepConfigTypeDef",
    "StepDetailTypeDef",
    "StepExecutionStatusDetailTypeDef",
    "StepStateChangeReasonTypeDef",
    "StepStatusTypeDef",
    "StepSummaryTypeDef",
    "StepTimelineTypeDef",
    "StepTypeDef",
    "StopNotebookExecutionInputRequestTypeDef",
    "StudioSummaryTypeDef",
    "StudioTypeDef",
    "SupportedProductConfigTypeDef",
    "TagTypeDef",
    "TerminateJobFlowsInputRequestTypeDef",
    "UpdateStudioInputRequestTypeDef",
    "UpdateStudioSessionMappingInputRequestTypeDef",
    "VolumeSpecificationTypeDef",
    "WaiterConfigTypeDef",
)

AddInstanceFleetInputRequestTypeDef = TypedDict(
    "AddInstanceFleetInputRequestTypeDef",
    {
        "ClusterId": str,
        "InstanceFleet": "InstanceFleetConfigTypeDef",
    },
)

AddInstanceFleetOutputTypeDef = TypedDict(
    "AddInstanceFleetOutputTypeDef",
    {
        "ClusterId": str,
        "InstanceFleetId": str,
        "ClusterArn": str,
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

AddInstanceGroupsInputRequestTypeDef = TypedDict(
    "AddInstanceGroupsInputRequestTypeDef",
    {
        "InstanceGroups": Sequence["InstanceGroupConfigTypeDef"],
        "JobFlowId": str,
    },
)

AddInstanceGroupsOutputTypeDef = TypedDict(
    "AddInstanceGroupsOutputTypeDef",
    {
        "JobFlowId": str,
        "InstanceGroupIds": List[str],
        "ClusterArn": str,
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

AddJobFlowStepsInputRequestTypeDef = TypedDict(
    "AddJobFlowStepsInputRequestTypeDef",
    {
        "JobFlowId": str,
        "Steps": Sequence["StepConfigTypeDef"],
    },
)

AddJobFlowStepsOutputTypeDef = TypedDict(
    "AddJobFlowStepsOutputTypeDef",
    {
        "StepIds": List[str],
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

AddTagsInputRequestTypeDef = TypedDict(
    "AddTagsInputRequestTypeDef",
    {
        "ResourceId": str,
        "Tags": Sequence["TagTypeDef"],
    },
)

ApplicationTypeDef = TypedDict(
    "ApplicationTypeDef",
    {
        "Name": NotRequired[str],
        "Version": NotRequired[str],
        "Args": NotRequired[List[str]],
        "AdditionalInfo": NotRequired[Dict[str, str]],
    },
)

AutoScalingPolicyDescriptionTypeDef = TypedDict(
    "AutoScalingPolicyDescriptionTypeDef",
    {
        "Status": NotRequired["AutoScalingPolicyStatusTypeDef"],
        "Constraints": NotRequired["ScalingConstraintsTypeDef"],
        "Rules": NotRequired[List["ScalingRuleTypeDef"]],
    },
)

AutoScalingPolicyStateChangeReasonTypeDef = TypedDict(
    "AutoScalingPolicyStateChangeReasonTypeDef",
    {
        "Code": NotRequired[AutoScalingPolicyStateChangeReasonCodeType],
        "Message": NotRequired[str],
    },
)

AutoScalingPolicyStatusTypeDef = TypedDict(
    "AutoScalingPolicyStatusTypeDef",
    {
        "State": NotRequired[AutoScalingPolicyStateType],
        "StateChangeReason": NotRequired["AutoScalingPolicyStateChangeReasonTypeDef"],
    },
)

AutoScalingPolicyTypeDef = TypedDict(
    "AutoScalingPolicyTypeDef",
    {
        "Constraints": "ScalingConstraintsTypeDef",
        "Rules": Sequence["ScalingRuleTypeDef"],
    },
)

AutoTerminationPolicyTypeDef = TypedDict(
    "AutoTerminationPolicyTypeDef",
    {
        "IdleTimeout": NotRequired[int],
    },
)

BlockPublicAccessConfigurationMetadataTypeDef = TypedDict(
    "BlockPublicAccessConfigurationMetadataTypeDef",
    {
        "CreationDateTime": datetime,
        "CreatedByArn": str,
    },
)

BlockPublicAccessConfigurationTypeDef = TypedDict(
    "BlockPublicAccessConfigurationTypeDef",
    {
        "BlockPublicSecurityGroupRules": bool,
        "PermittedPublicSecurityGroupRuleRanges": NotRequired[List["PortRangeTypeDef"]],
    },
)

BootstrapActionConfigTypeDef = TypedDict(
    "BootstrapActionConfigTypeDef",
    {
        "Name": str,
        "ScriptBootstrapAction": "ScriptBootstrapActionConfigTypeDef",
    },
)

BootstrapActionDetailTypeDef = TypedDict(
    "BootstrapActionDetailTypeDef",
    {
        "BootstrapActionConfig": NotRequired["BootstrapActionConfigTypeDef"],
    },
)

CancelStepsInfoTypeDef = TypedDict(
    "CancelStepsInfoTypeDef",
    {
        "StepId": NotRequired[str],
        "Status": NotRequired[CancelStepsRequestStatusType],
        "Reason": NotRequired[str],
    },
)

CancelStepsInputRequestTypeDef = TypedDict(
    "CancelStepsInputRequestTypeDef",
    {
        "ClusterId": str,
        "StepIds": Sequence[str],
        "StepCancellationOption": NotRequired[StepCancellationOptionType],
    },
)

CancelStepsOutputTypeDef = TypedDict(
    "CancelStepsOutputTypeDef",
    {
        "CancelStepsInfoList": List["CancelStepsInfoTypeDef"],
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

CloudWatchAlarmDefinitionTypeDef = TypedDict(
    "CloudWatchAlarmDefinitionTypeDef",
    {
        "ComparisonOperator": ComparisonOperatorType,
        "MetricName": str,
        "Period": int,
        "Threshold": float,
        "EvaluationPeriods": NotRequired[int],
        "Namespace": NotRequired[str],
        "Statistic": NotRequired[StatisticType],
        "Unit": NotRequired[UnitType],
        "Dimensions": NotRequired[Sequence["MetricDimensionTypeDef"]],
    },
)

ClusterStateChangeReasonTypeDef = TypedDict(
    "ClusterStateChangeReasonTypeDef",
    {
        "Code": NotRequired[ClusterStateChangeReasonCodeType],
        "Message": NotRequired[str],
    },
)

ClusterStatusTypeDef = TypedDict(
    "ClusterStatusTypeDef",
    {
        "State": NotRequired[ClusterStateType],
        "StateChangeReason": NotRequired["ClusterStateChangeReasonTypeDef"],
        "Timeline": NotRequired["ClusterTimelineTypeDef"],
    },
)

ClusterSummaryTypeDef = TypedDict(
    "ClusterSummaryTypeDef",
    {
        "Id": NotRequired[str],
        "Name": NotRequired[str],
        "Status": NotRequired["ClusterStatusTypeDef"],
        "NormalizedInstanceHours": NotRequired[int],
        "ClusterArn": NotRequired[str],
        "OutpostArn": NotRequired[str],
    },
)

ClusterTimelineTypeDef = TypedDict(
    "ClusterTimelineTypeDef",
    {
        "CreationDateTime": NotRequired[datetime],
        "ReadyDateTime": NotRequired[datetime],
        "EndDateTime": NotRequired[datetime],
    },
)

ClusterTypeDef = TypedDict(
    "ClusterTypeDef",
    {
        "Id": NotRequired[str],
        "Name": NotRequired[str],
        "Status": NotRequired["ClusterStatusTypeDef"],
        "Ec2InstanceAttributes": NotRequired["Ec2InstanceAttributesTypeDef"],
        "InstanceCollectionType": NotRequired[InstanceCollectionTypeType],
        "LogUri": NotRequired[str],
        "LogEncryptionKmsKeyId": NotRequired[str],
        "RequestedAmiVersion": NotRequired[str],
        "RunningAmiVersion": NotRequired[str],
        "ReleaseLabel": NotRequired[str],
        "AutoTerminate": NotRequired[bool],
        "TerminationProtected": NotRequired[bool],
        "VisibleToAllUsers": NotRequired[bool],
        "Applications": NotRequired[List["ApplicationTypeDef"]],
        "Tags": NotRequired[List["TagTypeDef"]],
        "ServiceRole": NotRequired[str],
        "NormalizedInstanceHours": NotRequired[int],
        "MasterPublicDnsName": NotRequired[str],
        "Configurations": NotRequired[List["ConfigurationTypeDef"]],
        "SecurityConfiguration": NotRequired[str],
        "AutoScalingRole": NotRequired[str],
        "ScaleDownBehavior": NotRequired[ScaleDownBehaviorType],
        "CustomAmiId": NotRequired[str],
        "EbsRootVolumeSize": NotRequired[int],
        "RepoUpgradeOnBoot": NotRequired[RepoUpgradeOnBootType],
        "KerberosAttributes": NotRequired["KerberosAttributesTypeDef"],
        "ClusterArn": NotRequired[str],
        "OutpostArn": NotRequired[str],
        "StepConcurrencyLevel": NotRequired[int],
        "PlacementGroups": NotRequired[List["PlacementGroupConfigTypeDef"]],
    },
)

CommandTypeDef = TypedDict(
    "CommandTypeDef",
    {
        "Name": NotRequired[str],
        "ScriptPath": NotRequired[str],
        "Args": NotRequired[List[str]],
    },
)

ComputeLimitsTypeDef = TypedDict(
    "ComputeLimitsTypeDef",
    {
        "UnitType": ComputeLimitsUnitTypeType,
        "MinimumCapacityUnits": int,
        "MaximumCapacityUnits": int,
        "MaximumOnDemandCapacityUnits": NotRequired[int],
        "MaximumCoreCapacityUnits": NotRequired[int],
    },
)

ConfigurationTypeDef = TypedDict(
    "ConfigurationTypeDef",
    {
        "Classification": NotRequired[str],
        "Configurations": NotRequired[Sequence[Dict[str, Any]]],
        "Properties": NotRequired[Mapping[str, str]],
    },
)

CreateSecurityConfigurationInputRequestTypeDef = TypedDict(
    "CreateSecurityConfigurationInputRequestTypeDef",
    {
        "Name": str,
        "SecurityConfiguration": str,
    },
)

CreateSecurityConfigurationOutputTypeDef = TypedDict(
    "CreateSecurityConfigurationOutputTypeDef",
    {
        "Name": str,
        "CreationDateTime": datetime,
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

CreateStudioInputRequestTypeDef = TypedDict(
    "CreateStudioInputRequestTypeDef",
    {
        "Name": str,
        "AuthMode": AuthModeType,
        "VpcId": str,
        "SubnetIds": Sequence[str],
        "ServiceRole": str,
        "WorkspaceSecurityGroupId": str,
        "EngineSecurityGroupId": str,
        "DefaultS3Location": str,
        "Description": NotRequired[str],
        "UserRole": NotRequired[str],
        "IdpAuthUrl": NotRequired[str],
        "IdpRelayStateParameterName": NotRequired[str],
        "Tags": NotRequired[Sequence["TagTypeDef"]],
    },
)

CreateStudioOutputTypeDef = TypedDict(
    "CreateStudioOutputTypeDef",
    {
        "StudioId": str,
        "Url": str,
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

CreateStudioSessionMappingInputRequestTypeDef = TypedDict(
    "CreateStudioSessionMappingInputRequestTypeDef",
    {
        "StudioId": str,
        "IdentityType": IdentityTypeType,
        "SessionPolicyArn": str,
        "IdentityId": NotRequired[str],
        "IdentityName": NotRequired[str],
    },
)

DeleteSecurityConfigurationInputRequestTypeDef = TypedDict(
    "DeleteSecurityConfigurationInputRequestTypeDef",
    {
        "Name": str,
    },
)

DeleteStudioInputRequestTypeDef = TypedDict(
    "DeleteStudioInputRequestTypeDef",
    {
        "StudioId": str,
    },
)

DeleteStudioSessionMappingInputRequestTypeDef = TypedDict(
    "DeleteStudioSessionMappingInputRequestTypeDef",
    {
        "StudioId": str,
        "IdentityType": IdentityTypeType,
        "IdentityId": NotRequired[str],
        "IdentityName": NotRequired[str],
    },
)

DescribeClusterInputRequestTypeDef = TypedDict(
    "DescribeClusterInputRequestTypeDef",
    {
        "ClusterId": str,
    },
)

DescribeClusterOutputTypeDef = TypedDict(
    "DescribeClusterOutputTypeDef",
    {
        "Cluster": "ClusterTypeDef",
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

DescribeJobFlowsInputRequestTypeDef = TypedDict(
    "DescribeJobFlowsInputRequestTypeDef",
    {
        "CreatedAfter": NotRequired[Union[datetime, str]],
        "CreatedBefore": NotRequired[Union[datetime, str]],
        "JobFlowIds": NotRequired[Sequence[str]],
        "JobFlowStates": NotRequired[Sequence[JobFlowExecutionStateType]],
    },
)

DescribeJobFlowsOutputTypeDef = TypedDict(
    "DescribeJobFlowsOutputTypeDef",
    {
        "JobFlows": List["JobFlowDetailTypeDef"],
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

DescribeNotebookExecutionInputRequestTypeDef = TypedDict(
    "DescribeNotebookExecutionInputRequestTypeDef",
    {
        "NotebookExecutionId": str,
    },
)

DescribeNotebookExecutionOutputTypeDef = TypedDict(
    "DescribeNotebookExecutionOutputTypeDef",
    {
        "NotebookExecution": "NotebookExecutionTypeDef",
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

DescribeReleaseLabelInputRequestTypeDef = TypedDict(
    "DescribeReleaseLabelInputRequestTypeDef",
    {
        "ReleaseLabel": NotRequired[str],
        "NextToken": NotRequired[str],
        "MaxResults": NotRequired[int],
    },
)

DescribeReleaseLabelOutputTypeDef = TypedDict(
    "DescribeReleaseLabelOutputTypeDef",
    {
        "ReleaseLabel": str,
        "Applications": List["SimplifiedApplicationTypeDef"],
        "NextToken": str,
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

DescribeSecurityConfigurationInputRequestTypeDef = TypedDict(
    "DescribeSecurityConfigurationInputRequestTypeDef",
    {
        "Name": str,
    },
)

DescribeSecurityConfigurationOutputTypeDef = TypedDict(
    "DescribeSecurityConfigurationOutputTypeDef",
    {
        "Name": str,
        "SecurityConfiguration": str,
        "CreationDateTime": datetime,
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

DescribeStepInputRequestTypeDef = TypedDict(
    "DescribeStepInputRequestTypeDef",
    {
        "ClusterId": str,
        "StepId": str,
    },
)

DescribeStepOutputTypeDef = TypedDict(
    "DescribeStepOutputTypeDef",
    {
        "Step": "StepTypeDef",
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

DescribeStudioInputRequestTypeDef = TypedDict(
    "DescribeStudioInputRequestTypeDef",
    {
        "StudioId": str,
    },
)

DescribeStudioOutputTypeDef = TypedDict(
    "DescribeStudioOutputTypeDef",
    {
        "Studio": "StudioTypeDef",
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

EbsBlockDeviceConfigTypeDef = TypedDict(
    "EbsBlockDeviceConfigTypeDef",
    {
        "VolumeSpecification": "VolumeSpecificationTypeDef",
        "VolumesPerInstance": NotRequired[int],
    },
)

EbsBlockDeviceTypeDef = TypedDict(
    "EbsBlockDeviceTypeDef",
    {
        "VolumeSpecification": NotRequired["VolumeSpecificationTypeDef"],
        "Device": NotRequired[str],
    },
)

EbsConfigurationTypeDef = TypedDict(
    "EbsConfigurationTypeDef",
    {
        "EbsBlockDeviceConfigs": NotRequired[Sequence["EbsBlockDeviceConfigTypeDef"]],
        "EbsOptimized": NotRequired[bool],
    },
)

EbsVolumeTypeDef = TypedDict(
    "EbsVolumeTypeDef",
    {
        "Device": NotRequired[str],
        "VolumeId": NotRequired[str],
    },
)

Ec2InstanceAttributesTypeDef = TypedDict(
    "Ec2InstanceAttributesTypeDef",
    {
        "Ec2KeyName": NotRequired[str],
        "Ec2SubnetId": NotRequired[str],
        "RequestedEc2SubnetIds": NotRequired[List[str]],
        "Ec2AvailabilityZone": NotRequired[str],
        "RequestedEc2AvailabilityZones": NotRequired[List[str]],
        "IamInstanceProfile": NotRequired[str],
        "EmrManagedMasterSecurityGroup": NotRequired[str],
        "EmrManagedSlaveSecurityGroup": NotRequired[str],
        "ServiceAccessSecurityGroup": NotRequired[str],
        "AdditionalMasterSecurityGroups": NotRequired[List[str]],
        "AdditionalSlaveSecurityGroups": NotRequired[List[str]],
    },
)

ExecutionEngineConfigTypeDef = TypedDict(
    "ExecutionEngineConfigTypeDef",
    {
        "Id": str,
        "Type": NotRequired[Literal["EMR"]],
        "MasterInstanceSecurityGroupId": NotRequired[str],
    },
)

FailureDetailsTypeDef = TypedDict(
    "FailureDetailsTypeDef",
    {
        "Reason": NotRequired[str],
        "Message": NotRequired[str],
        "LogFile": NotRequired[str],
    },
)

GetAutoTerminationPolicyInputRequestTypeDef = TypedDict(
    "GetAutoTerminationPolicyInputRequestTypeDef",
    {
        "ClusterId": str,
    },
)

GetAutoTerminationPolicyOutputTypeDef = TypedDict(
    "GetAutoTerminationPolicyOutputTypeDef",
    {
        "AutoTerminationPolicy": "AutoTerminationPolicyTypeDef",
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

GetBlockPublicAccessConfigurationOutputTypeDef = TypedDict(
    "GetBlockPublicAccessConfigurationOutputTypeDef",
    {
        "BlockPublicAccessConfiguration": "BlockPublicAccessConfigurationTypeDef",
        "BlockPublicAccessConfigurationMetadata": "BlockPublicAccessConfigurationMetadataTypeDef",
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

GetManagedScalingPolicyInputRequestTypeDef = TypedDict(
    "GetManagedScalingPolicyInputRequestTypeDef",
    {
        "ClusterId": str,
    },
)

GetManagedScalingPolicyOutputTypeDef = TypedDict(
    "GetManagedScalingPolicyOutputTypeDef",
    {
        "ManagedScalingPolicy": "ManagedScalingPolicyTypeDef",
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

GetStudioSessionMappingInputRequestTypeDef = TypedDict(
    "GetStudioSessionMappingInputRequestTypeDef",
    {
        "StudioId": str,
        "IdentityType": IdentityTypeType,
        "IdentityId": NotRequired[str],
        "IdentityName": NotRequired[str],
    },
)

GetStudioSessionMappingOutputTypeDef = TypedDict(
    "GetStudioSessionMappingOutputTypeDef",
    {
        "SessionMapping": "SessionMappingDetailTypeDef",
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

HadoopJarStepConfigTypeDef = TypedDict(
    "HadoopJarStepConfigTypeDef",
    {
        "Jar": str,
        "Properties": NotRequired[Sequence["KeyValueTypeDef"]],
        "MainClass": NotRequired[str],
        "Args": NotRequired[Sequence[str]],
    },
)

HadoopStepConfigTypeDef = TypedDict(
    "HadoopStepConfigTypeDef",
    {
        "Jar": NotRequired[str],
        "Properties": NotRequired[Dict[str, str]],
        "MainClass": NotRequired[str],
        "Args": NotRequired[List[str]],
    },
)

InstanceFleetConfigTypeDef = TypedDict(
    "InstanceFleetConfigTypeDef",
    {
        "InstanceFleetType": InstanceFleetTypeType,
        "Name": NotRequired[str],
        "TargetOnDemandCapacity": NotRequired[int],
        "TargetSpotCapacity": NotRequired[int],
        "InstanceTypeConfigs": NotRequired[Sequence["InstanceTypeConfigTypeDef"]],
        "LaunchSpecifications": NotRequired["InstanceFleetProvisioningSpecificationsTypeDef"],
    },
)

InstanceFleetModifyConfigTypeDef = TypedDict(
    "InstanceFleetModifyConfigTypeDef",
    {
        "InstanceFleetId": str,
        "TargetOnDemandCapacity": NotRequired[int],
        "TargetSpotCapacity": NotRequired[int],
    },
)

InstanceFleetProvisioningSpecificationsTypeDef = TypedDict(
    "InstanceFleetProvisioningSpecificationsTypeDef",
    {
        "SpotSpecification": NotRequired["SpotProvisioningSpecificationTypeDef"],
        "OnDemandSpecification": NotRequired["OnDemandProvisioningSpecificationTypeDef"],
    },
)

InstanceFleetStateChangeReasonTypeDef = TypedDict(
    "InstanceFleetStateChangeReasonTypeDef",
    {
        "Code": NotRequired[InstanceFleetStateChangeReasonCodeType],
        "Message": NotRequired[str],
    },
)

InstanceFleetStatusTypeDef = TypedDict(
    "InstanceFleetStatusTypeDef",
    {
        "State": NotRequired[InstanceFleetStateType],
        "StateChangeReason": NotRequired["InstanceFleetStateChangeReasonTypeDef"],
        "Timeline": NotRequired["InstanceFleetTimelineTypeDef"],
    },
)

InstanceFleetTimelineTypeDef = TypedDict(
    "InstanceFleetTimelineTypeDef",
    {
        "CreationDateTime": NotRequired[datetime],
        "ReadyDateTime": NotRequired[datetime],
        "EndDateTime": NotRequired[datetime],
    },
)

InstanceFleetTypeDef = TypedDict(
    "InstanceFleetTypeDef",
    {
        "Id": NotRequired[str],
        "Name": NotRequired[str],
        "Status": NotRequired["InstanceFleetStatusTypeDef"],
        "InstanceFleetType": NotRequired[InstanceFleetTypeType],
        "TargetOnDemandCapacity": NotRequired[int],
        "TargetSpotCapacity": NotRequired[int],
        "ProvisionedOnDemandCapacity": NotRequired[int],
        "ProvisionedSpotCapacity": NotRequired[int],
        "InstanceTypeSpecifications": NotRequired[List["InstanceTypeSpecificationTypeDef"]],
        "LaunchSpecifications": NotRequired["InstanceFleetProvisioningSpecificationsTypeDef"],
    },
)

InstanceGroupConfigTypeDef = TypedDict(
    "InstanceGroupConfigTypeDef",
    {
        "InstanceRole": InstanceRoleTypeType,
        "InstanceType": str,
        "InstanceCount": int,
        "Name": NotRequired[str],
        "Market": NotRequired[MarketTypeType],
        "BidPrice": NotRequired[str],
        "Configurations": NotRequired[Sequence["ConfigurationTypeDef"]],
        "EbsConfiguration": NotRequired["EbsConfigurationTypeDef"],
        "AutoScalingPolicy": NotRequired["AutoScalingPolicyTypeDef"],
        "CustomAmiId": NotRequired[str],
    },
)

InstanceGroupDetailTypeDef = TypedDict(
    "InstanceGroupDetailTypeDef",
    {
        "Market": MarketTypeType,
        "InstanceRole": InstanceRoleTypeType,
        "InstanceType": str,
        "InstanceRequestCount": int,
        "InstanceRunningCount": int,
        "State": InstanceGroupStateType,
        "CreationDateTime": datetime,
        "InstanceGroupId": NotRequired[str],
        "Name": NotRequired[str],
        "BidPrice": NotRequired[str],
        "LastStateChangeReason": NotRequired[str],
        "StartDateTime": NotRequired[datetime],
        "ReadyDateTime": NotRequired[datetime],
        "EndDateTime": NotRequired[datetime],
        "CustomAmiId": NotRequired[str],
    },
)

InstanceGroupModifyConfigTypeDef = TypedDict(
    "InstanceGroupModifyConfigTypeDef",
    {
        "InstanceGroupId": str,
        "InstanceCount": NotRequired[int],
        "EC2InstanceIdsToTerminate": NotRequired[Sequence[str]],
        "ShrinkPolicy": NotRequired["ShrinkPolicyTypeDef"],
        "Configurations": NotRequired[Sequence["ConfigurationTypeDef"]],
    },
)

InstanceGroupStateChangeReasonTypeDef = TypedDict(
    "InstanceGroupStateChangeReasonTypeDef",
    {
        "Code": NotRequired[InstanceGroupStateChangeReasonCodeType],
        "Message": NotRequired[str],
    },
)

InstanceGroupStatusTypeDef = TypedDict(
    "InstanceGroupStatusTypeDef",
    {
        "State": NotRequired[InstanceGroupStateType],
        "StateChangeReason": NotRequired["InstanceGroupStateChangeReasonTypeDef"],
        "Timeline": NotRequired["InstanceGroupTimelineTypeDef"],
    },
)

InstanceGroupTimelineTypeDef = TypedDict(
    "InstanceGroupTimelineTypeDef",
    {
        "CreationDateTime": NotRequired[datetime],
        "ReadyDateTime": NotRequired[datetime],
        "EndDateTime": NotRequired[datetime],
    },
)

InstanceGroupTypeDef = TypedDict(
    "InstanceGroupTypeDef",
    {
        "Id": NotRequired[str],
        "Name": NotRequired[str],
        "Market": NotRequired[MarketTypeType],
        "InstanceGroupType": NotRequired[InstanceGroupTypeType],
        "BidPrice": NotRequired[str],
        "InstanceType": NotRequired[str],
        "RequestedInstanceCount": NotRequired[int],
        "RunningInstanceCount": NotRequired[int],
        "Status": NotRequired["InstanceGroupStatusTypeDef"],
        "Configurations": NotRequired[List["ConfigurationTypeDef"]],
        "ConfigurationsVersion": NotRequired[int],
        "LastSuccessfullyAppliedConfigurations": NotRequired[List["ConfigurationTypeDef"]],
        "LastSuccessfullyAppliedConfigurationsVersion": NotRequired[int],
        "EbsBlockDevices": NotRequired[List["EbsBlockDeviceTypeDef"]],
        "EbsOptimized": NotRequired[bool],
        "ShrinkPolicy": NotRequired["ShrinkPolicyTypeDef"],
        "AutoScalingPolicy": NotRequired["AutoScalingPolicyDescriptionTypeDef"],
        "CustomAmiId": NotRequired[str],
    },
)

InstanceResizePolicyTypeDef = TypedDict(
    "InstanceResizePolicyTypeDef",
    {
        "InstancesToTerminate": NotRequired[List[str]],
        "InstancesToProtect": NotRequired[List[str]],
        "InstanceTerminationTimeout": NotRequired[int],
    },
)

InstanceStateChangeReasonTypeDef = TypedDict(
    "InstanceStateChangeReasonTypeDef",
    {
        "Code": NotRequired[InstanceStateChangeReasonCodeType],
        "Message": NotRequired[str],
    },
)

InstanceStatusTypeDef = TypedDict(
    "InstanceStatusTypeDef",
    {
        "State": NotRequired[InstanceStateType],
        "StateChangeReason": NotRequired["InstanceStateChangeReasonTypeDef"],
        "Timeline": NotRequired["InstanceTimelineTypeDef"],
    },
)

InstanceTimelineTypeDef = TypedDict(
    "InstanceTimelineTypeDef",
    {
        "CreationDateTime": NotRequired[datetime],
        "ReadyDateTime": NotRequired[datetime],
        "EndDateTime": NotRequired[datetime],
    },
)

InstanceTypeConfigTypeDef = TypedDict(
    "InstanceTypeConfigTypeDef",
    {
        "InstanceType": str,
        "WeightedCapacity": NotRequired[int],
        "BidPrice": NotRequired[str],
        "BidPriceAsPercentageOfOnDemandPrice": NotRequired[float],
        "EbsConfiguration": NotRequired["EbsConfigurationTypeDef"],
        "Configurations": NotRequired[Sequence["ConfigurationTypeDef"]],
        "CustomAmiId": NotRequired[str],
    },
)

InstanceTypeDef = TypedDict(
    "InstanceTypeDef",
    {
        "Id": NotRequired[str],
        "Ec2InstanceId": NotRequired[str],
        "PublicDnsName": NotRequired[str],
        "PublicIpAddress": NotRequired[str],
        "PrivateDnsName": NotRequired[str],
        "PrivateIpAddress": NotRequired[str],
        "Status": NotRequired["InstanceStatusTypeDef"],
        "InstanceGroupId": NotRequired[str],
        "InstanceFleetId": NotRequired[str],
        "Market": NotRequired[MarketTypeType],
        "InstanceType": NotRequired[str],
        "EbsVolumes": NotRequired[List["EbsVolumeTypeDef"]],
    },
)

InstanceTypeSpecificationTypeDef = TypedDict(
    "InstanceTypeSpecificationTypeDef",
    {
        "InstanceType": NotRequired[str],
        "WeightedCapacity": NotRequired[int],
        "BidPrice": NotRequired[str],
        "BidPriceAsPercentageOfOnDemandPrice": NotRequired[float],
        "Configurations": NotRequired[List["ConfigurationTypeDef"]],
        "EbsBlockDevices": NotRequired[List["EbsBlockDeviceTypeDef"]],
        "EbsOptimized": NotRequired[bool],
        "CustomAmiId": NotRequired[str],
    },
)

JobFlowDetailTypeDef = TypedDict(
    "JobFlowDetailTypeDef",
    {
        "JobFlowId": str,
        "Name": str,
        "ExecutionStatusDetail": "JobFlowExecutionStatusDetailTypeDef",
        "Instances": "JobFlowInstancesDetailTypeDef",
        "LogUri": NotRequired[str],
        "LogEncryptionKmsKeyId": NotRequired[str],
        "AmiVersion": NotRequired[str],
        "Steps": NotRequired[List["StepDetailTypeDef"]],
        "BootstrapActions": NotRequired[List["BootstrapActionDetailTypeDef"]],
        "SupportedProducts": NotRequired[List[str]],
        "VisibleToAllUsers": NotRequired[bool],
        "JobFlowRole": NotRequired[str],
        "ServiceRole": NotRequired[str],
        "AutoScalingRole": NotRequired[str],
        "ScaleDownBehavior": NotRequired[ScaleDownBehaviorType],
    },
)

JobFlowExecutionStatusDetailTypeDef = TypedDict(
    "JobFlowExecutionStatusDetailTypeDef",
    {
        "State": JobFlowExecutionStateType,
        "CreationDateTime": datetime,
        "StartDateTime": NotRequired[datetime],
        "ReadyDateTime": NotRequired[datetime],
        "EndDateTime": NotRequired[datetime],
        "LastStateChangeReason": NotRequired[str],
    },
)

JobFlowInstancesConfigTypeDef = TypedDict(
    "JobFlowInstancesConfigTypeDef",
    {
        "MasterInstanceType": NotRequired[str],
        "SlaveInstanceType": NotRequired[str],
        "InstanceCount": NotRequired[int],
        "InstanceGroups": NotRequired[Sequence["InstanceGroupConfigTypeDef"]],
        "InstanceFleets": NotRequired[Sequence["InstanceFleetConfigTypeDef"]],
        "Ec2KeyName": NotRequired[str],
        "Placement": NotRequired["PlacementTypeTypeDef"],
        "KeepJobFlowAliveWhenNoSteps": NotRequired[bool],
        "TerminationProtected": NotRequired[bool],
        "HadoopVersion": NotRequired[str],
        "Ec2SubnetId": NotRequired[str],
        "Ec2SubnetIds": NotRequired[Sequence[str]],
        "EmrManagedMasterSecurityGroup": NotRequired[str],
        "EmrManagedSlaveSecurityGroup": NotRequired[str],
        "ServiceAccessSecurityGroup": NotRequired[str],
        "AdditionalMasterSecurityGroups": NotRequired[Sequence[str]],
        "AdditionalSlaveSecurityGroups": NotRequired[Sequence[str]],
    },
)

JobFlowInstancesDetailTypeDef = TypedDict(
    "JobFlowInstancesDetailTypeDef",
    {
        "MasterInstanceType": str,
        "SlaveInstanceType": str,
        "InstanceCount": int,
        "MasterPublicDnsName": NotRequired[str],
        "MasterInstanceId": NotRequired[str],
        "InstanceGroups": NotRequired[List["InstanceGroupDetailTypeDef"]],
        "NormalizedInstanceHours": NotRequired[int],
        "Ec2KeyName": NotRequired[str],
        "Ec2SubnetId": NotRequired[str],
        "Placement": NotRequired["PlacementTypeTypeDef"],
        "KeepJobFlowAliveWhenNoSteps": NotRequired[bool],
        "TerminationProtected": NotRequired[bool],
        "HadoopVersion": NotRequired[str],
    },
)

KerberosAttributesTypeDef = TypedDict(
    "KerberosAttributesTypeDef",
    {
        "Realm": str,
        "KdcAdminPassword": str,
        "CrossRealmTrustPrincipalPassword": NotRequired[str],
        "ADDomainJoinUser": NotRequired[str],
        "ADDomainJoinPassword": NotRequired[str],
    },
)

KeyValueTypeDef = TypedDict(
    "KeyValueTypeDef",
    {
        "Key": NotRequired[str],
        "Value": NotRequired[str],
    },
)

ListBootstrapActionsInputRequestTypeDef = TypedDict(
    "ListBootstrapActionsInputRequestTypeDef",
    {
        "ClusterId": str,
        "Marker": NotRequired[str],
    },
)

ListBootstrapActionsOutputTypeDef = TypedDict(
    "ListBootstrapActionsOutputTypeDef",
    {
        "BootstrapActions": List["CommandTypeDef"],
        "Marker": str,
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

ListClustersInputRequestTypeDef = TypedDict(
    "ListClustersInputRequestTypeDef",
    {
        "CreatedAfter": NotRequired[Union[datetime, str]],
        "CreatedBefore": NotRequired[Union[datetime, str]],
        "ClusterStates": NotRequired[Sequence[ClusterStateType]],
        "Marker": NotRequired[str],
    },
)

ListClustersOutputTypeDef = TypedDict(
    "ListClustersOutputTypeDef",
    {
        "Clusters": List["ClusterSummaryTypeDef"],
        "Marker": str,
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

ListInstanceFleetsInputRequestTypeDef = TypedDict(
    "ListInstanceFleetsInputRequestTypeDef",
    {
        "ClusterId": str,
        "Marker": NotRequired[str],
    },
)

ListInstanceFleetsOutputTypeDef = TypedDict(
    "ListInstanceFleetsOutputTypeDef",
    {
        "InstanceFleets": List["InstanceFleetTypeDef"],
        "Marker": str,
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

ListInstanceGroupsInputRequestTypeDef = TypedDict(
    "ListInstanceGroupsInputRequestTypeDef",
    {
        "ClusterId": str,
        "Marker": NotRequired[str],
    },
)

ListInstanceGroupsOutputTypeDef = TypedDict(
    "ListInstanceGroupsOutputTypeDef",
    {
        "InstanceGroups": List["InstanceGroupTypeDef"],
        "Marker": str,
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

ListInstancesInputRequestTypeDef = TypedDict(
    "ListInstancesInputRequestTypeDef",
    {
        "ClusterId": str,
        "InstanceGroupId": NotRequired[str],
        "InstanceGroupTypes": NotRequired[Sequence[InstanceGroupTypeType]],
        "InstanceFleetId": NotRequired[str],
        "InstanceFleetType": NotRequired[InstanceFleetTypeType],
        "InstanceStates": NotRequired[Sequence[InstanceStateType]],
        "Marker": NotRequired[str],
    },
)

ListInstancesOutputTypeDef = TypedDict(
    "ListInstancesOutputTypeDef",
    {
        "Instances": List["InstanceTypeDef"],
        "Marker": str,
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

ListNotebookExecutionsInputRequestTypeDef = TypedDict(
    "ListNotebookExecutionsInputRequestTypeDef",
    {
        "EditorId": NotRequired[str],
        "Status": NotRequired[NotebookExecutionStatusType],
        "From": NotRequired[Union[datetime, str]],
        "To": NotRequired[Union[datetime, str]],
        "Marker": NotRequired[str],
    },
)

ListNotebookExecutionsOutputTypeDef = TypedDict(
    "ListNotebookExecutionsOutputTypeDef",
    {
        "NotebookExecutions": List["NotebookExecutionSummaryTypeDef"],
        "Marker": str,
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

ListReleaseLabelsInputRequestTypeDef = TypedDict(
    "ListReleaseLabelsInputRequestTypeDef",
    {
        "Filters": NotRequired["ReleaseLabelFilterTypeDef"],
        "NextToken": NotRequired[str],
        "MaxResults": NotRequired[int],
    },
)

ListReleaseLabelsOutputTypeDef = TypedDict(
    "ListReleaseLabelsOutputTypeDef",
    {
        "ReleaseLabels": List[str],
        "NextToken": str,
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

ListSecurityConfigurationsInputRequestTypeDef = TypedDict(
    "ListSecurityConfigurationsInputRequestTypeDef",
    {
        "Marker": NotRequired[str],
    },
)

ListSecurityConfigurationsOutputTypeDef = TypedDict(
    "ListSecurityConfigurationsOutputTypeDef",
    {
        "SecurityConfigurations": List["SecurityConfigurationSummaryTypeDef"],
        "Marker": str,
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

ListStepsInputRequestTypeDef = TypedDict(
    "ListStepsInputRequestTypeDef",
    {
        "ClusterId": str,
        "StepStates": NotRequired[Sequence[StepStateType]],
        "StepIds": NotRequired[Sequence[str]],
        "Marker": NotRequired[str],
    },
)

ListStepsOutputTypeDef = TypedDict(
    "ListStepsOutputTypeDef",
    {
        "Steps": List["StepSummaryTypeDef"],
        "Marker": str,
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

ListStudioSessionMappingsInputRequestTypeDef = TypedDict(
    "ListStudioSessionMappingsInputRequestTypeDef",
    {
        "StudioId": NotRequired[str],
        "IdentityType": NotRequired[IdentityTypeType],
        "Marker": NotRequired[str],
    },
)

ListStudioSessionMappingsOutputTypeDef = TypedDict(
    "ListStudioSessionMappingsOutputTypeDef",
    {
        "SessionMappings": List["SessionMappingSummaryTypeDef"],
        "Marker": str,
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

ListStudiosInputRequestTypeDef = TypedDict(
    "ListStudiosInputRequestTypeDef",
    {
        "Marker": NotRequired[str],
    },
)

ListStudiosOutputTypeDef = TypedDict(
    "ListStudiosOutputTypeDef",
    {
        "Studios": List["StudioSummaryTypeDef"],
        "Marker": str,
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

ManagedScalingPolicyTypeDef = TypedDict(
    "ManagedScalingPolicyTypeDef",
    {
        "ComputeLimits": NotRequired["ComputeLimitsTypeDef"],
    },
)

MetricDimensionTypeDef = TypedDict(
    "MetricDimensionTypeDef",
    {
        "Key": NotRequired[str],
        "Value": NotRequired[str],
    },
)

ModifyClusterInputRequestTypeDef = TypedDict(
    "ModifyClusterInputRequestTypeDef",
    {
        "ClusterId": str,
        "StepConcurrencyLevel": NotRequired[int],
    },
)

ModifyClusterOutputTypeDef = TypedDict(
    "ModifyClusterOutputTypeDef",
    {
        "StepConcurrencyLevel": int,
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

ModifyInstanceFleetInputRequestTypeDef = TypedDict(
    "ModifyInstanceFleetInputRequestTypeDef",
    {
        "ClusterId": str,
        "InstanceFleet": "InstanceFleetModifyConfigTypeDef",
    },
)

ModifyInstanceGroupsInputRequestTypeDef = TypedDict(
    "ModifyInstanceGroupsInputRequestTypeDef",
    {
        "ClusterId": NotRequired[str],
        "InstanceGroups": NotRequired[Sequence["InstanceGroupModifyConfigTypeDef"]],
    },
)

NotebookExecutionSummaryTypeDef = TypedDict(
    "NotebookExecutionSummaryTypeDef",
    {
        "NotebookExecutionId": NotRequired[str],
        "EditorId": NotRequired[str],
        "NotebookExecutionName": NotRequired[str],
        "Status": NotRequired[NotebookExecutionStatusType],
        "StartTime": NotRequired[datetime],
        "EndTime": NotRequired[datetime],
    },
)

NotebookExecutionTypeDef = TypedDict(
    "NotebookExecutionTypeDef",
    {
        "NotebookExecutionId": NotRequired[str],
        "EditorId": NotRequired[str],
        "ExecutionEngine": NotRequired["ExecutionEngineConfigTypeDef"],
        "NotebookExecutionName": NotRequired[str],
        "NotebookParams": NotRequired[str],
        "Status": NotRequired[NotebookExecutionStatusType],
        "StartTime": NotRequired[datetime],
        "EndTime": NotRequired[datetime],
        "Arn": NotRequired[str],
        "OutputNotebookURI": NotRequired[str],
        "LastStateChangeReason": NotRequired[str],
        "NotebookInstanceSecurityGroupId": NotRequired[str],
        "Tags": NotRequired[List["TagTypeDef"]],
    },
)

OnDemandCapacityReservationOptionsTypeDef = TypedDict(
    "OnDemandCapacityReservationOptionsTypeDef",
    {
        "UsageStrategy": NotRequired[Literal["use-capacity-reservations-first"]],
        "CapacityReservationPreference": NotRequired[OnDemandCapacityReservationPreferenceType],
        "CapacityReservationResourceGroupArn": NotRequired[str],
    },
)

OnDemandProvisioningSpecificationTypeDef = TypedDict(
    "OnDemandProvisioningSpecificationTypeDef",
    {
        "AllocationStrategy": Literal["lowest-price"],
        "CapacityReservationOptions": NotRequired["OnDemandCapacityReservationOptionsTypeDef"],
    },
)

PaginatorConfigTypeDef = TypedDict(
    "PaginatorConfigTypeDef",
    {
        "MaxItems": NotRequired[int],
        "PageSize": NotRequired[int],
        "StartingToken": NotRequired[str],
    },
)

PlacementGroupConfigTypeDef = TypedDict(
    "PlacementGroupConfigTypeDef",
    {
        "InstanceRole": InstanceRoleTypeType,
        "PlacementStrategy": NotRequired[PlacementGroupStrategyType],
    },
)

PlacementTypeTypeDef = TypedDict(
    "PlacementTypeTypeDef",
    {
        "AvailabilityZone": NotRequired[str],
        "AvailabilityZones": NotRequired[List[str]],
    },
)

PortRangeTypeDef = TypedDict(
    "PortRangeTypeDef",
    {
        "MinRange": int,
        "MaxRange": NotRequired[int],
    },
)

PutAutoScalingPolicyInputRequestTypeDef = TypedDict(
    "PutAutoScalingPolicyInputRequestTypeDef",
    {
        "ClusterId": str,
        "InstanceGroupId": str,
        "AutoScalingPolicy": "AutoScalingPolicyTypeDef",
    },
)

PutAutoScalingPolicyOutputTypeDef = TypedDict(
    "PutAutoScalingPolicyOutputTypeDef",
    {
        "ClusterId": str,
        "InstanceGroupId": str,
        "AutoScalingPolicy": "AutoScalingPolicyDescriptionTypeDef",
        "ClusterArn": str,
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

PutAutoTerminationPolicyInputRequestTypeDef = TypedDict(
    "PutAutoTerminationPolicyInputRequestTypeDef",
    {
        "ClusterId": str,
        "AutoTerminationPolicy": NotRequired["AutoTerminationPolicyTypeDef"],
    },
)

PutBlockPublicAccessConfigurationInputRequestTypeDef = TypedDict(
    "PutBlockPublicAccessConfigurationInputRequestTypeDef",
    {
        "BlockPublicAccessConfiguration": "BlockPublicAccessConfigurationTypeDef",
    },
)

PutManagedScalingPolicyInputRequestTypeDef = TypedDict(
    "PutManagedScalingPolicyInputRequestTypeDef",
    {
        "ClusterId": str,
        "ManagedScalingPolicy": "ManagedScalingPolicyTypeDef",
    },
)

ReleaseLabelFilterTypeDef = TypedDict(
    "ReleaseLabelFilterTypeDef",
    {
        "Prefix": NotRequired[str],
        "Application": NotRequired[str],
    },
)

RemoveAutoScalingPolicyInputRequestTypeDef = TypedDict(
    "RemoveAutoScalingPolicyInputRequestTypeDef",
    {
        "ClusterId": str,
        "InstanceGroupId": str,
    },
)

RemoveAutoTerminationPolicyInputRequestTypeDef = TypedDict(
    "RemoveAutoTerminationPolicyInputRequestTypeDef",
    {
        "ClusterId": str,
    },
)

RemoveManagedScalingPolicyInputRequestTypeDef = TypedDict(
    "RemoveManagedScalingPolicyInputRequestTypeDef",
    {
        "ClusterId": str,
    },
)

RemoveTagsInputRequestTypeDef = TypedDict(
    "RemoveTagsInputRequestTypeDef",
    {
        "ResourceId": str,
        "TagKeys": Sequence[str],
    },
)

ResponseMetadataTypeDef = TypedDict(
    "ResponseMetadataTypeDef",
    {
        "RequestId": str,
        "HostId": str,
        "HTTPStatusCode": int,
        "HTTPHeaders": Dict[str, str],
        "RetryAttempts": int,
    },
)

RunJobFlowInputRequestTypeDef = TypedDict(
    "RunJobFlowInputRequestTypeDef",
    {
        "Name": str,
        "Instances": "JobFlowInstancesConfigTypeDef",
        "LogUri": NotRequired[str],
        "LogEncryptionKmsKeyId": NotRequired[str],
        "AdditionalInfo": NotRequired[str],
        "AmiVersion": NotRequired[str],
        "ReleaseLabel": NotRequired[str],
        "Steps": NotRequired[Sequence["StepConfigTypeDef"]],
        "BootstrapActions": NotRequired[Sequence["BootstrapActionConfigTypeDef"]],
        "SupportedProducts": NotRequired[Sequence[str]],
        "NewSupportedProducts": NotRequired[Sequence["SupportedProductConfigTypeDef"]],
        "Applications": NotRequired[Sequence["ApplicationTypeDef"]],
        "Configurations": NotRequired[Sequence["ConfigurationTypeDef"]],
        "VisibleToAllUsers": NotRequired[bool],
        "JobFlowRole": NotRequired[str],
        "ServiceRole": NotRequired[str],
        "Tags": NotRequired[Sequence["TagTypeDef"]],
        "SecurityConfiguration": NotRequired[str],
        "AutoScalingRole": NotRequired[str],
        "ScaleDownBehavior": NotRequired[ScaleDownBehaviorType],
        "CustomAmiId": NotRequired[str],
        "EbsRootVolumeSize": NotRequired[int],
        "RepoUpgradeOnBoot": NotRequired[RepoUpgradeOnBootType],
        "KerberosAttributes": NotRequired["KerberosAttributesTypeDef"],
        "StepConcurrencyLevel": NotRequired[int],
        "ManagedScalingPolicy": NotRequired["ManagedScalingPolicyTypeDef"],
        "PlacementGroupConfigs": NotRequired[Sequence["PlacementGroupConfigTypeDef"]],
        "AutoTerminationPolicy": NotRequired["AutoTerminationPolicyTypeDef"],
    },
)

RunJobFlowOutputTypeDef = TypedDict(
    "RunJobFlowOutputTypeDef",
    {
        "JobFlowId": str,
        "ClusterArn": str,
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

ScalingActionTypeDef = TypedDict(
    "ScalingActionTypeDef",
    {
        "SimpleScalingPolicyConfiguration": "SimpleScalingPolicyConfigurationTypeDef",
        "Market": NotRequired[MarketTypeType],
    },
)

ScalingConstraintsTypeDef = TypedDict(
    "ScalingConstraintsTypeDef",
    {
        "MinCapacity": int,
        "MaxCapacity": int,
    },
)

ScalingRuleTypeDef = TypedDict(
    "ScalingRuleTypeDef",
    {
        "Name": str,
        "Action": "ScalingActionTypeDef",
        "Trigger": "ScalingTriggerTypeDef",
        "Description": NotRequired[str],
    },
)

ScalingTriggerTypeDef = TypedDict(
    "ScalingTriggerTypeDef",
    {
        "CloudWatchAlarmDefinition": "CloudWatchAlarmDefinitionTypeDef",
    },
)

ScriptBootstrapActionConfigTypeDef = TypedDict(
    "ScriptBootstrapActionConfigTypeDef",
    {
        "Path": str,
        "Args": NotRequired[List[str]],
    },
)

SecurityConfigurationSummaryTypeDef = TypedDict(
    "SecurityConfigurationSummaryTypeDef",
    {
        "Name": NotRequired[str],
        "CreationDateTime": NotRequired[datetime],
    },
)

SessionMappingDetailTypeDef = TypedDict(
    "SessionMappingDetailTypeDef",
    {
        "StudioId": NotRequired[str],
        "IdentityId": NotRequired[str],
        "IdentityName": NotRequired[str],
        "IdentityType": NotRequired[IdentityTypeType],
        "SessionPolicyArn": NotRequired[str],
        "CreationTime": NotRequired[datetime],
        "LastModifiedTime": NotRequired[datetime],
    },
)

SessionMappingSummaryTypeDef = TypedDict(
    "SessionMappingSummaryTypeDef",
    {
        "StudioId": NotRequired[str],
        "IdentityId": NotRequired[str],
        "IdentityName": NotRequired[str],
        "IdentityType": NotRequired[IdentityTypeType],
        "SessionPolicyArn": NotRequired[str],
        "CreationTime": NotRequired[datetime],
    },
)

SetTerminationProtectionInputRequestTypeDef = TypedDict(
    "SetTerminationProtectionInputRequestTypeDef",
    {
        "JobFlowIds": Sequence[str],
        "TerminationProtected": bool,
    },
)

SetVisibleToAllUsersInputRequestTypeDef = TypedDict(
    "SetVisibleToAllUsersInputRequestTypeDef",
    {
        "JobFlowIds": Sequence[str],
        "VisibleToAllUsers": bool,
    },
)

ShrinkPolicyTypeDef = TypedDict(
    "ShrinkPolicyTypeDef",
    {
        "DecommissionTimeout": NotRequired[int],
        "InstanceResizePolicy": NotRequired["InstanceResizePolicyTypeDef"],
    },
)

SimpleScalingPolicyConfigurationTypeDef = TypedDict(
    "SimpleScalingPolicyConfigurationTypeDef",
    {
        "ScalingAdjustment": int,
        "AdjustmentType": NotRequired[AdjustmentTypeType],
        "CoolDown": NotRequired[int],
    },
)

SimplifiedApplicationTypeDef = TypedDict(
    "SimplifiedApplicationTypeDef",
    {
        "Name": NotRequired[str],
        "Version": NotRequired[str],
    },
)

SpotProvisioningSpecificationTypeDef = TypedDict(
    "SpotProvisioningSpecificationTypeDef",
    {
        "TimeoutDurationMinutes": int,
        "TimeoutAction": SpotProvisioningTimeoutActionType,
        "BlockDurationMinutes": NotRequired[int],
        "AllocationStrategy": NotRequired[Literal["capacity-optimized"]],
    },
)

StartNotebookExecutionInputRequestTypeDef = TypedDict(
    "StartNotebookExecutionInputRequestTypeDef",
    {
        "EditorId": str,
        "RelativePath": str,
        "ExecutionEngine": "ExecutionEngineConfigTypeDef",
        "ServiceRole": str,
        "NotebookExecutionName": NotRequired[str],
        "NotebookParams": NotRequired[str],
        "NotebookInstanceSecurityGroupId": NotRequired[str],
        "Tags": NotRequired[Sequence["TagTypeDef"]],
    },
)

StartNotebookExecutionOutputTypeDef = TypedDict(
    "StartNotebookExecutionOutputTypeDef",
    {
        "NotebookExecutionId": str,
        "ResponseMetadata": "ResponseMetadataTypeDef",
    },
)

StepConfigTypeDef = TypedDict(
    "StepConfigTypeDef",
    {
        "Name": str,
        "HadoopJarStep": "HadoopJarStepConfigTypeDef",
        "ActionOnFailure": NotRequired[ActionOnFailureType],
    },
)

StepDetailTypeDef = TypedDict(
    "StepDetailTypeDef",
    {
        "StepConfig": "StepConfigTypeDef",
        "ExecutionStatusDetail": "StepExecutionStatusDetailTypeDef",
    },
)

StepExecutionStatusDetailTypeDef = TypedDict(
    "StepExecutionStatusDetailTypeDef",
    {
        "State": StepExecutionStateType,
        "CreationDateTime": datetime,
        "StartDateTime": NotRequired[datetime],
        "EndDateTime": NotRequired[datetime],
        "LastStateChangeReason": NotRequired[str],
    },
)

StepStateChangeReasonTypeDef = TypedDict(
    "StepStateChangeReasonTypeDef",
    {
        "Code": NotRequired[Literal["NONE"]],
        "Message": NotRequired[str],
    },
)

StepStatusTypeDef = TypedDict(
    "StepStatusTypeDef",
    {
        "State": NotRequired[StepStateType],
        "StateChangeReason": NotRequired["StepStateChangeReasonTypeDef"],
        "FailureDetails": NotRequired["FailureDetailsTypeDef"],
        "Timeline": NotRequired["StepTimelineTypeDef"],
    },
)

StepSummaryTypeDef = TypedDict(
    "StepSummaryTypeDef",
    {
        "Id": NotRequired[str],
        "Name": NotRequired[str],
        "Config": NotRequired["HadoopStepConfigTypeDef"],
        "ActionOnFailure": NotRequired[ActionOnFailureType],
        "Status": NotRequired["StepStatusTypeDef"],
    },
)

StepTimelineTypeDef = TypedDict(
    "StepTimelineTypeDef",
    {
        "CreationDateTime": NotRequired[datetime],
        "StartDateTime": NotRequired[datetime],
        "EndDateTime": NotRequired[datetime],
    },
)

StepTypeDef = TypedDict(
    "StepTypeDef",
    {
        "Id": NotRequired[str],
        "Name": NotRequired[str],
        "Config": NotRequired["HadoopStepConfigTypeDef"],
        "ActionOnFailure": NotRequired[ActionOnFailureType],
        "Status": NotRequired["StepStatusTypeDef"],
    },
)

StopNotebookExecutionInputRequestTypeDef = TypedDict(
    "StopNotebookExecutionInputRequestTypeDef",
    {
        "NotebookExecutionId": str,
    },
)

StudioSummaryTypeDef = TypedDict(
    "StudioSummaryTypeDef",
    {
        "StudioId": NotRequired[str],
        "Name": NotRequired[str],
        "VpcId": NotRequired[str],
        "Description": NotRequired[str],
        "Url": NotRequired[str],
        "AuthMode": NotRequired[AuthModeType],
        "CreationTime": NotRequired[datetime],
    },
)

StudioTypeDef = TypedDict(
    "StudioTypeDef",
    {
        "StudioId": NotRequired[str],
        "StudioArn": NotRequired[str],
        "Name": NotRequired[str],
        "Description": NotRequired[str],
        "AuthMode": NotRequired[AuthModeType],
        "VpcId": NotRequired[str],
        "SubnetIds": NotRequired[List[str]],
        "ServiceRole": NotRequired[str],
        "UserRole": NotRequired[str],
        "WorkspaceSecurityGroupId": NotRequired[str],
        "EngineSecurityGroupId": NotRequired[str],
        "Url": NotRequired[str],
        "CreationTime": NotRequired[datetime],
        "DefaultS3Location": NotRequired[str],
        "IdpAuthUrl": NotRequired[str],
        "IdpRelayStateParameterName": NotRequired[str],
        "Tags": NotRequired[List["TagTypeDef"]],
    },
)

SupportedProductConfigTypeDef = TypedDict(
    "SupportedProductConfigTypeDef",
    {
        "Name": NotRequired[str],
        "Args": NotRequired[Sequence[str]],
    },
)

TagTypeDef = TypedDict(
    "TagTypeDef",
    {
        "Key": NotRequired[str],
        "Value": NotRequired[str],
    },
)

TerminateJobFlowsInputRequestTypeDef = TypedDict(
    "TerminateJobFlowsInputRequestTypeDef",
    {
        "JobFlowIds": Sequence[str],
    },
)

UpdateStudioInputRequestTypeDef = TypedDict(
    "UpdateStudioInputRequestTypeDef",
    {
        "StudioId": str,
        "Name": NotRequired[str],
        "Description": NotRequired[str],
        "SubnetIds": NotRequired[Sequence[str]],
        "DefaultS3Location": NotRequired[str],
    },
)

UpdateStudioSessionMappingInputRequestTypeDef = TypedDict(
    "UpdateStudioSessionMappingInputRequestTypeDef",
    {
        "StudioId": str,
        "IdentityType": IdentityTypeType,
        "SessionPolicyArn": str,
        "IdentityId": NotRequired[str],
        "IdentityName": NotRequired[str],
    },
)

VolumeSpecificationTypeDef = TypedDict(
    "VolumeSpecificationTypeDef",
    {
        "VolumeType": str,
        "SizeInGB": int,
        "Iops": NotRequired[int],
    },
)

WaiterConfigTypeDef = TypedDict(
    "WaiterConfigTypeDef",
    {
        "Delay": NotRequired[int],
        "MaxAttempts": NotRequired[int],
    },
)



"""
Type annotations for emr service client.

[Open documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html)

Usage::

    ```python
    from aiobotocore.session import get_session
    from types_aiobotocore_emr.client import EMRClient

    session = get_session()
    async with session.create_client("emr") as client:
        client: EMRClient
    ```
"""
import sys
from datetime import datetime
from typing import Any, Dict, Mapping, Sequence, Type, Union, overload

from aiobotocore.client import AioBaseClient
from botocore.client import ClientMeta

from .literals import (
    AuthModeType,
    ClusterStateType,
    IdentityTypeType,
    InstanceFleetTypeType,
    InstanceGroupTypeType,
    InstanceStateType,
    JobFlowExecutionStateType,
    NotebookExecutionStatusType,
    RepoUpgradeOnBootType,
    ScaleDownBehaviorType,
    StepCancellationOptionType,
    StepStateType,
)
from .paginator import (
    ListBootstrapActionsPaginator,
    ListClustersPaginator,
    ListInstanceFleetsPaginator,
    ListInstanceGroupsPaginator,
    ListInstancesPaginator,
    ListNotebookExecutionsPaginator,
    ListSecurityConfigurationsPaginator,
    ListStepsPaginator,
    ListStudioSessionMappingsPaginator,
    ListStudiosPaginator,
)
from .type_defs import (
    AddInstanceFleetOutputTypeDef,
    AddInstanceGroupsOutputTypeDef,
    AddJobFlowStepsOutputTypeDef,
    ApplicationTypeDef,
    AutoScalingPolicyTypeDef,
    AutoTerminationPolicyTypeDef,
    BlockPublicAccessConfigurationTypeDef,
    BootstrapActionConfigTypeDef,
    CancelStepsOutputTypeDef,
    ConfigurationTypeDef,
    CreateSecurityConfigurationOutputTypeDef,
    CreateStudioOutputTypeDef,
    DescribeClusterOutputTypeDef,
    DescribeJobFlowsOutputTypeDef,
    DescribeNotebookExecutionOutputTypeDef,
    DescribeReleaseLabelOutputTypeDef,
    DescribeSecurityConfigurationOutputTypeDef,
    DescribeStepOutputTypeDef,
    DescribeStudioOutputTypeDef,
    ExecutionEngineConfigTypeDef,
    GetAutoTerminationPolicyOutputTypeDef,
    GetBlockPublicAccessConfigurationOutputTypeDef,
    GetManagedScalingPolicyOutputTypeDef,
    GetStudioSessionMappingOutputTypeDef,
    InstanceFleetConfigTypeDef,
    InstanceFleetModifyConfigTypeDef,
    InstanceGroupConfigTypeDef,
    InstanceGroupModifyConfigTypeDef,
    JobFlowInstancesConfigTypeDef,
    KerberosAttributesTypeDef,
    ListBootstrapActionsOutputTypeDef,
    ListClustersOutputTypeDef,
    ListInstanceFleetsOutputTypeDef,
    ListInstanceGroupsOutputTypeDef,
    ListInstancesOutputTypeDef,
    ListNotebookExecutionsOutputTypeDef,
    ListReleaseLabelsOutputTypeDef,
    ListSecurityConfigurationsOutputTypeDef,
    ListStepsOutputTypeDef,
    ListStudioSessionMappingsOutputTypeDef,
    ListStudiosOutputTypeDef,
    ManagedScalingPolicyTypeDef,
    ModifyClusterOutputTypeDef,
    PlacementGroupConfigTypeDef,
    PutAutoScalingPolicyOutputTypeDef,
    ReleaseLabelFilterTypeDef,
    RunJobFlowOutputTypeDef,
    StartNotebookExecutionOutputTypeDef,
    StepConfigTypeDef,
    SupportedProductConfigTypeDef,
    TagTypeDef,
)
from .waiter import ClusterRunningWaiter, ClusterTerminatedWaiter, StepCompleteWaiter

if sys.version_info >= (3, 9):
    from typing import Literal
else:
    from typing_extensions import Literal


__all__ = ("EMRClient",)


class BotocoreClientError(BaseException):
    MSG_TEMPLATE: str

    def __init__(self, error_response: Mapping[str, Any], operation_name: str) -> None:
        self.response: Dict[str, Any]
        self.operation_name: str


class Exceptions:
    ClientError: Type[BotocoreClientError]
    InternalServerError: Type[BotocoreClientError]
    InternalServerException: Type[BotocoreClientError]
    InvalidRequestException: Type[BotocoreClientError]


class EMRClient(AioBaseClient):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client)
    [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html)
    """

    meta: ClientMeta

    @property
    def exceptions(self) -> Exceptions:
        """
        EMRClient exceptions.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.exceptions)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#exceptions)
        """

    async def add_instance_fleet(
        self, *, ClusterId: str, InstanceFleet: "InstanceFleetConfigTypeDef"
    ) -> AddInstanceFleetOutputTypeDef:
        """
        Adds an instance fleet to a running cluster.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.add_instance_fleet)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#add_instance_fleet)
        """

    async def add_instance_groups(
        self, *, InstanceGroups: Sequence["InstanceGroupConfigTypeDef"], JobFlowId: str
    ) -> AddInstanceGroupsOutputTypeDef:
        """
        Adds one or more instance groups to a running cluster.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.add_instance_groups)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#add_instance_groups)
        """

    async def add_job_flow_steps(
        self, *, JobFlowId: str, Steps: Sequence["StepConfigTypeDef"]
    ) -> AddJobFlowStepsOutputTypeDef:
        """
        AddJobFlowSteps adds new steps to a running cluster.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.add_job_flow_steps)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#add_job_flow_steps)
        """

    async def add_tags(self, *, ResourceId: str, Tags: Sequence["TagTypeDef"]) -> Dict[str, Any]:
        """
        Adds tags to an Amazon EMR resource, such as a cluster or an Amazon EMR Studio.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.add_tags)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#add_tags)
        """

    def can_paginate(self, operation_name: str) -> bool:
        """
        Check if an operation can be paginated.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.can_paginate)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#can_paginate)
        """

    async def cancel_steps(
        self,
        *,
        ClusterId: str,
        StepIds: Sequence[str],
        StepCancellationOption: StepCancellationOptionType = ...
    ) -> CancelStepsOutputTypeDef:
        """
        Cancels a pending step or steps in a running cluster.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.cancel_steps)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#cancel_steps)
        """

    async def create_security_configuration(
        self, *, Name: str, SecurityConfiguration: str
    ) -> CreateSecurityConfigurationOutputTypeDef:
        """
        Creates a security configuration, which is stored in the service and can be
        specified when a cluster is created.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.create_security_configuration)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#create_security_configuration)
        """

    async def create_studio(
        self,
        *,
        Name: str,
        AuthMode: AuthModeType,
        VpcId: str,
        SubnetIds: Sequence[str],
        ServiceRole: str,
        WorkspaceSecurityGroupId: str,
        EngineSecurityGroupId: str,
        DefaultS3Location: str,
        Description: str = ...,
        UserRole: str = ...,
        IdpAuthUrl: str = ...,
        IdpRelayStateParameterName: str = ...,
        Tags: Sequence["TagTypeDef"] = ...
    ) -> CreateStudioOutputTypeDef:
        """
        Creates a new Amazon EMR Studio.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.create_studio)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#create_studio)
        """

    async def create_studio_session_mapping(
        self,
        *,
        StudioId: str,
        IdentityType: IdentityTypeType,
        SessionPolicyArn: str,
        IdentityId: str = ...,
        IdentityName: str = ...
    ) -> None:
        """
        Maps a user or group to the Amazon EMR Studio specified by `StudioId` , and
        applies a session policy to refine Studio permissions for that user or group.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.create_studio_session_mapping)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#create_studio_session_mapping)
        """

    async def delete_security_configuration(self, *, Name: str) -> Dict[str, Any]:
        """
        Deletes a security configuration.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.delete_security_configuration)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#delete_security_configuration)
        """

    async def delete_studio(self, *, StudioId: str) -> None:
        """
        Removes an Amazon EMR Studio from the Studio metadata store.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.delete_studio)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#delete_studio)
        """

    async def delete_studio_session_mapping(
        self,
        *,
        StudioId: str,
        IdentityType: IdentityTypeType,
        IdentityId: str = ...,
        IdentityName: str = ...
    ) -> None:
        """
        Removes a user or group from an Amazon EMR Studio.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.delete_studio_session_mapping)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#delete_studio_session_mapping)
        """

    async def describe_cluster(self, *, ClusterId: str) -> DescribeClusterOutputTypeDef:
        """
        Provides cluster-level details including status, hardware and software
        configuration, VPC settings, and so on.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.describe_cluster)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#describe_cluster)
        """

    async def describe_job_flows(
        self,
        *,
        CreatedAfter: Union[datetime, str] = ...,
        CreatedBefore: Union[datetime, str] = ...,
        JobFlowIds: Sequence[str] = ...,
        JobFlowStates: Sequence[JobFlowExecutionStateType] = ...
    ) -> DescribeJobFlowsOutputTypeDef:
        """
        This API is no longer supported and will eventually be removed.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.describe_job_flows)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#describe_job_flows)
        """

    async def describe_notebook_execution(
        self, *, NotebookExecutionId: str
    ) -> DescribeNotebookExecutionOutputTypeDef:
        """
        Provides details of a notebook execution.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.describe_notebook_execution)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#describe_notebook_execution)
        """

    async def describe_release_label(
        self, *, ReleaseLabel: str = ..., NextToken: str = ..., MaxResults: int = ...
    ) -> DescribeReleaseLabelOutputTypeDef:
        """
        Provides EMR release label details, such as releases available the region where
        the API request is run, and the available applications for a specific EMR
        release label.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.describe_release_label)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#describe_release_label)
        """

    async def describe_security_configuration(
        self, *, Name: str
    ) -> DescribeSecurityConfigurationOutputTypeDef:
        """
        Provides the details of a security configuration by returning the configuration
        JSON.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.describe_security_configuration)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#describe_security_configuration)
        """

    async def describe_step(self, *, ClusterId: str, StepId: str) -> DescribeStepOutputTypeDef:
        """
        Provides more detail about the cluster step.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.describe_step)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#describe_step)
        """

    async def describe_studio(self, *, StudioId: str) -> DescribeStudioOutputTypeDef:
        """
        Returns details for the specified Amazon EMR Studio including ID, Name, VPC,
        Studio access URL, and so on.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.describe_studio)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#describe_studio)
        """

    async def generate_presigned_url(
        self,
        ClientMethod: str,
        Params: Mapping[str, Any] = ...,
        ExpiresIn: int = 3600,
        HttpMethod: str = ...,
    ) -> str:
        """
        Generate a presigned url given a client, its method, and arguments.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.generate_presigned_url)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#generate_presigned_url)
        """

    async def get_auto_termination_policy(
        self, *, ClusterId: str
    ) -> GetAutoTerminationPolicyOutputTypeDef:
        """
        Returns the auto-termination policy for an Amazon EMR cluster.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.get_auto_termination_policy)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#get_auto_termination_policy)
        """

    async def get_block_public_access_configuration(
        self,
    ) -> GetBlockPublicAccessConfigurationOutputTypeDef:
        """
        Returns the Amazon EMR block public access configuration for your Amazon Web
        Services account in the current Region.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.get_block_public_access_configuration)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#get_block_public_access_configuration)
        """

    async def get_managed_scaling_policy(
        self, *, ClusterId: str
    ) -> GetManagedScalingPolicyOutputTypeDef:
        """
        Fetches the attached managed scaling policy for an Amazon EMR cluster.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.get_managed_scaling_policy)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#get_managed_scaling_policy)
        """

    async def get_studio_session_mapping(
        self,
        *,
        StudioId: str,
        IdentityType: IdentityTypeType,
        IdentityId: str = ...,
        IdentityName: str = ...
    ) -> GetStudioSessionMappingOutputTypeDef:
        """
        Fetches mapping details for the specified Amazon EMR Studio and identity (user
        or group).

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.get_studio_session_mapping)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#get_studio_session_mapping)
        """

    async def list_bootstrap_actions(
        self, *, ClusterId: str, Marker: str = ...
    ) -> ListBootstrapActionsOutputTypeDef:
        """
        Provides information about the bootstrap actions associated with a cluster.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.list_bootstrap_actions)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#list_bootstrap_actions)
        """

    async def list_clusters(
        self,
        *,
        CreatedAfter: Union[datetime, str] = ...,
        CreatedBefore: Union[datetime, str] = ...,
        ClusterStates: Sequence[ClusterStateType] = ...,
        Marker: str = ...
    ) -> ListClustersOutputTypeDef:
        """
        Provides the status of all clusters visible to this Amazon Web Services account.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.list_clusters)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#list_clusters)
        """

    async def list_instance_fleets(
        self, *, ClusterId: str, Marker: str = ...
    ) -> ListInstanceFleetsOutputTypeDef:
        """
        Lists all available details about the instance fleets in a cluster.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.list_instance_fleets)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#list_instance_fleets)
        """

    async def list_instance_groups(
        self, *, ClusterId: str, Marker: str = ...
    ) -> ListInstanceGroupsOutputTypeDef:
        """
        Provides all available details about the instance groups in a cluster.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.list_instance_groups)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#list_instance_groups)
        """

    async def list_instances(
        self,
        *,
        ClusterId: str,
        InstanceGroupId: str = ...,
        InstanceGroupTypes: Sequence[InstanceGroupTypeType] = ...,
        InstanceFleetId: str = ...,
        InstanceFleetType: InstanceFleetTypeType = ...,
        InstanceStates: Sequence[InstanceStateType] = ...,
        Marker: str = ...
    ) -> ListInstancesOutputTypeDef:
        """
        Provides information for all active EC2 instances and EC2 instances terminated
        in the last 30 days, up to a maximum of 2,000.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.list_instances)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#list_instances)
        """

    async def list_notebook_executions(
        self,
        *,
        EditorId: str = ...,
        Status: NotebookExecutionStatusType = ...,
        From: Union[datetime, str] = ...,
        To: Union[datetime, str] = ...,
        Marker: str = ...
    ) -> ListNotebookExecutionsOutputTypeDef:
        """
        Provides summaries of all notebook executions.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.list_notebook_executions)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#list_notebook_executions)
        """

    async def list_release_labels(
        self,
        *,
        Filters: "ReleaseLabelFilterTypeDef" = ...,
        NextToken: str = ...,
        MaxResults: int = ...
    ) -> ListReleaseLabelsOutputTypeDef:
        """
        Retrieves release labels of EMR services in the region where the API is called.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.list_release_labels)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#list_release_labels)
        """

    async def list_security_configurations(
        self, *, Marker: str = ...
    ) -> ListSecurityConfigurationsOutputTypeDef:
        """
        Lists all the security configurations visible to this account, providing their
        creation dates and times, and their names.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.list_security_configurations)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#list_security_configurations)
        """

    async def list_steps(
        self,
        *,
        ClusterId: str,
        StepStates: Sequence[StepStateType] = ...,
        StepIds: Sequence[str] = ...,
        Marker: str = ...
    ) -> ListStepsOutputTypeDef:
        """
        Provides a list of steps for the cluster in reverse order unless you specify
        `stepIds` with the request or filter by `StepStates`.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.list_steps)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#list_steps)
        """

    async def list_studio_session_mappings(
        self, *, StudioId: str = ..., IdentityType: IdentityTypeType = ..., Marker: str = ...
    ) -> ListStudioSessionMappingsOutputTypeDef:
        """
        Returns a list of all user or group session mappings for the Amazon EMR Studio
        specified by `StudioId` .

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.list_studio_session_mappings)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#list_studio_session_mappings)
        """

    async def list_studios(self, *, Marker: str = ...) -> ListStudiosOutputTypeDef:
        """
        Returns a list of all Amazon EMR Studios associated with the Amazon Web Services
        account.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.list_studios)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#list_studios)
        """

    async def modify_cluster(
        self, *, ClusterId: str, StepConcurrencyLevel: int = ...
    ) -> ModifyClusterOutputTypeDef:
        """
        Modifies the number of steps that can be executed concurrently for the cluster
        specified using ClusterID.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.modify_cluster)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#modify_cluster)
        """

    async def modify_instance_fleet(
        self, *, ClusterId: str, InstanceFleet: "InstanceFleetModifyConfigTypeDef"
    ) -> None:
        """
        Modifies the target On-Demand and target Spot capacities for the instance fleet
        with the specified InstanceFleetID within the cluster specified using ClusterID.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.modify_instance_fleet)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#modify_instance_fleet)
        """

    async def modify_instance_groups(
        self,
        *,
        ClusterId: str = ...,
        InstanceGroups: Sequence["InstanceGroupModifyConfigTypeDef"] = ...
    ) -> None:
        """
        ModifyInstanceGroups modifies the number of nodes and configuration settings of
        an instance group.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.modify_instance_groups)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#modify_instance_groups)
        """

    async def put_auto_scaling_policy(
        self, *, ClusterId: str, InstanceGroupId: str, AutoScalingPolicy: "AutoScalingPolicyTypeDef"
    ) -> PutAutoScalingPolicyOutputTypeDef:
        """
        Creates or updates an automatic scaling policy for a core instance group or task
        instance group in an Amazon EMR cluster.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.put_auto_scaling_policy)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#put_auto_scaling_policy)
        """

    async def put_auto_termination_policy(
        self, *, ClusterId: str, AutoTerminationPolicy: "AutoTerminationPolicyTypeDef" = ...
    ) -> Dict[str, Any]:
        """
        .

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.put_auto_termination_policy)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#put_auto_termination_policy)
        """

    async def put_block_public_access_configuration(
        self, *, BlockPublicAccessConfiguration: "BlockPublicAccessConfigurationTypeDef"
    ) -> Dict[str, Any]:
        """
        Creates or updates an Amazon EMR block public access configuration for your
        Amazon Web Services account in the current Region.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.put_block_public_access_configuration)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#put_block_public_access_configuration)
        """

    async def put_managed_scaling_policy(
        self, *, ClusterId: str, ManagedScalingPolicy: "ManagedScalingPolicyTypeDef"
    ) -> Dict[str, Any]:
        """
        Creates or updates a managed scaling policy for an Amazon EMR cluster.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.put_managed_scaling_policy)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#put_managed_scaling_policy)
        """

    async def remove_auto_scaling_policy(
        self, *, ClusterId: str, InstanceGroupId: str
    ) -> Dict[str, Any]:
        """
        Removes an automatic scaling policy from a specified instance group within an
        EMR cluster.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.remove_auto_scaling_policy)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#remove_auto_scaling_policy)
        """

    async def remove_auto_termination_policy(self, *, ClusterId: str) -> Dict[str, Any]:
        """
        Removes an auto-termination policy from an Amazon EMR cluster.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.remove_auto_termination_policy)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#remove_auto_termination_policy)
        """

    async def remove_managed_scaling_policy(self, *, ClusterId: str) -> Dict[str, Any]:
        """
        Removes a managed scaling policy from a specified EMR cluster.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.remove_managed_scaling_policy)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#remove_managed_scaling_policy)
        """

    async def remove_tags(self, *, ResourceId: str, TagKeys: Sequence[str]) -> Dict[str, Any]:
        """
        Removes tags from an Amazon EMR resource, such as a cluster or Amazon EMR
        Studio.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.remove_tags)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#remove_tags)
        """

    async def run_job_flow(
        self,
        *,
        Name: str,
        Instances: "JobFlowInstancesConfigTypeDef",
        LogUri: str = ...,
        LogEncryptionKmsKeyId: str = ...,
        AdditionalInfo: str = ...,
        AmiVersion: str = ...,
        ReleaseLabel: str = ...,
        Steps: Sequence["StepConfigTypeDef"] = ...,
        BootstrapActions: Sequence["BootstrapActionConfigTypeDef"] = ...,
        SupportedProducts: Sequence[str] = ...,
        NewSupportedProducts: Sequence["SupportedProductConfigTypeDef"] = ...,
        Applications: Sequence["ApplicationTypeDef"] = ...,
        Configurations: Sequence["ConfigurationTypeDef"] = ...,
        VisibleToAllUsers: bool = ...,
        JobFlowRole: str = ...,
        ServiceRole: str = ...,
        Tags: Sequence["TagTypeDef"] = ...,
        SecurityConfiguration: str = ...,
        AutoScalingRole: str = ...,
        ScaleDownBehavior: ScaleDownBehaviorType = ...,
        CustomAmiId: str = ...,
        EbsRootVolumeSize: int = ...,
        RepoUpgradeOnBoot: RepoUpgradeOnBootType = ...,
        KerberosAttributes: "KerberosAttributesTypeDef" = ...,
        StepConcurrencyLevel: int = ...,
        ManagedScalingPolicy: "ManagedScalingPolicyTypeDef" = ...,
        PlacementGroupConfigs: Sequence["PlacementGroupConfigTypeDef"] = ...,
        AutoTerminationPolicy: "AutoTerminationPolicyTypeDef" = ...
    ) -> RunJobFlowOutputTypeDef:
        """
        RunJobFlow creates and starts running a new cluster (job flow).

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.run_job_flow)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#run_job_flow)
        """

    async def set_termination_protection(
        self, *, JobFlowIds: Sequence[str], TerminationProtected: bool
    ) -> None:
        """
        SetTerminationProtection locks a cluster (job flow) so the EC2 instances in the
        cluster cannot be terminated by user intervention, an API call, or in the event
        of a job-flow error.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.set_termination_protection)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#set_termination_protection)
        """

    async def set_visible_to_all_users(
        self, *, JobFlowIds: Sequence[str], VisibleToAllUsers: bool
    ) -> None:
        """
        .

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.set_visible_to_all_users)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#set_visible_to_all_users)
        """

    async def start_notebook_execution(
        self,
        *,
        EditorId: str,
        RelativePath: str,
        ExecutionEngine: "ExecutionEngineConfigTypeDef",
        ServiceRole: str,
        NotebookExecutionName: str = ...,
        NotebookParams: str = ...,
        NotebookInstanceSecurityGroupId: str = ...,
        Tags: Sequence["TagTypeDef"] = ...
    ) -> StartNotebookExecutionOutputTypeDef:
        """
        Starts a notebook execution.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.start_notebook_execution)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#start_notebook_execution)
        """

    async def stop_notebook_execution(self, *, NotebookExecutionId: str) -> None:
        """
        Stops a notebook execution.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.stop_notebook_execution)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#stop_notebook_execution)
        """

    async def terminate_job_flows(self, *, JobFlowIds: Sequence[str]) -> None:
        """
        TerminateJobFlows shuts a list of clusters (job flows) down.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.terminate_job_flows)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#terminate_job_flows)
        """

    async def update_studio(
        self,
        *,
        StudioId: str,
        Name: str = ...,
        Description: str = ...,
        SubnetIds: Sequence[str] = ...,
        DefaultS3Location: str = ...
    ) -> None:
        """
        Updates an Amazon EMR Studio configuration, including attributes such as name,
        description, and subnets.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.update_studio)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#update_studio)
        """

    async def update_studio_session_mapping(
        self,
        *,
        StudioId: str,
        IdentityType: IdentityTypeType,
        SessionPolicyArn: str,
        IdentityId: str = ...,
        IdentityName: str = ...
    ) -> None:
        """
        Updates the session policy attached to the user or group for the specified
        Amazon EMR Studio.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.update_studio_session_mapping)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#update_studio_session_mapping)
        """

    @overload
    def get_paginator(
        self, operation_name: Literal["list_bootstrap_actions"]
    ) -> ListBootstrapActionsPaginator:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.get_paginator)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#get_paginator)
        """

    @overload
    def get_paginator(self, operation_name: Literal["list_clusters"]) -> ListClustersPaginator:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.get_paginator)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#get_paginator)
        """

    @overload
    def get_paginator(
        self, operation_name: Literal["list_instance_fleets"]
    ) -> ListInstanceFleetsPaginator:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.get_paginator)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#get_paginator)
        """

    @overload
    def get_paginator(
        self, operation_name: Literal["list_instance_groups"]
    ) -> ListInstanceGroupsPaginator:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.get_paginator)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#get_paginator)
        """

    @overload
    def get_paginator(self, operation_name: Literal["list_instances"]) -> ListInstancesPaginator:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.get_paginator)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#get_paginator)
        """

    @overload
    def get_paginator(
        self, operation_name: Literal["list_notebook_executions"]
    ) -> ListNotebookExecutionsPaginator:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.get_paginator)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#get_paginator)
        """

    @overload
    def get_paginator(
        self, operation_name: Literal["list_security_configurations"]
    ) -> ListSecurityConfigurationsPaginator:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.get_paginator)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#get_paginator)
        """

    @overload
    def get_paginator(self, operation_name: Literal["list_steps"]) -> ListStepsPaginator:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.get_paginator)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#get_paginator)
        """

    @overload
    def get_paginator(
        self, operation_name: Literal["list_studio_session_mappings"]
    ) -> ListStudioSessionMappingsPaginator:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.get_paginator)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#get_paginator)
        """

    @overload
    def get_paginator(self, operation_name: Literal["list_studios"]) -> ListStudiosPaginator:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.get_paginator)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#get_paginator)
        """

    @overload
    def get_waiter(self, waiter_name: Literal["cluster_running"]) -> ClusterRunningWaiter:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.get_waiter)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#get_waiter)
        """

    @overload
    def get_waiter(self, waiter_name: Literal["cluster_terminated"]) -> ClusterTerminatedWaiter:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.get_waiter)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#get_waiter)
        """

    @overload
    def get_waiter(self, waiter_name: Literal["step_complete"]) -> StepCompleteWaiter:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client.get_waiter)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html#get_waiter)
        """

    async def __aenter__(self) -> "EMRClient":
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html)
        """

    async def __aexit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> Any:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr.html#EMR.Client)
        [Show types-aiobotocore documentation](https://vemel.github.io/types_aiobotocore_docs/types_aiobotocore_emr/client.html)
        """
